
declare option ddtek:xml-streaming 'yes';
declare option ddtek:serialize "indent=yes, omit-xml-declaration=no";

(: external reference to XML document :)
declare variable $xPRSXMLInput as document-node() external;

declare variable $logURL := "log.xml";
declare function local:addNode($error as element()) as element()*
{
	let $errLog := if(fn:doc-available($logURL)) then
					doc($logURL)/Errors
				   else ()
	let $newLog :=
			<Errors
				xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
				xsi:noNamespaceSchemaLocation="ErrorNodes.xsd">
				{
					$errLog/*,
					$error
				}
			</Errors>
	
	return ( 
		ddtek:serialize-to-url($newLog, $logURL,""),
		$newLog
	)
};


(: Retrieves the size of a white space or red/gold space from a secondary file and inserts it into the node :)
declare function local:replaceElms($n as element(), $acctRecNum as xs:string) as element()* 
{
	local:addNode(
		<MESSAGE_WHITE_SPACE>{
			concat("The white space size (for VISA region 01B) for &quot;", 
			$acctRecNum, 
			".xml&quot; could not be found. Please check the xDoc output.")
		}</MESSAGE_WHITE_SPACE>
	)/*
};

(: Retrieves the size of the message from a secondary file and inserts it into the message node :)
declare function local:processMessageNode($n as element(), $acctRecNum as xs:string) as element()* 
{
	local:addNode(
	<MESSAGE_ERROR>
		{$n/MESSAGE_TOOL_PK}
		<MESSAGE_MESSAGE>{
			concat("The message size (for VISA region 01A) for &quot;", 
				$acctRecNum, "_", 
				$n/MESSAGE_TOOL_PK/text(), 
				".xml&quot; could not be found. Please check the xDoc output.")
		}</MESSAGE_MESSAGE>
	</MESSAGE_ERROR>
	)/*
};


(: Calls subfunctions to add specific nodes or data values :)
declare function local:updateNodes($n as element(), $recNumber as xs:string) as element()*
{
	if ((local-name($n) eq 'WHITE_SPACE_SIZE') or (local-name($n) eq 'RED_GOLD_SIZE')) then 
		local:replaceElms($n, $recNumber)
	else if (local-name($n) eq 'MESSAGE') then 
		local:processMessageNode($n, $recNumber)
	else 
		element { name($n) } 
		{	
			(
				$n/@*, 
				$n/text(),
				for $child in $n/*
				return local:updateNodes($child, $recNumber)
			)
		}
	
};


<VISA_DATA 
	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
	xsi:noNamespaceSchemaLocation="VISA_DATA.xsd">
{
	for $acctRec in $xPRSXMLInput/VISA_DATA/ACCOUNT_RECORD
	let $acctRecNumber := $acctRec/ACCOUNT_RECORD_KEY_PK/text()
	return local:updateNodes($acctRec, $acctRecNumber)
}
</VISA_DATA>
(: Stylus Studio meta-information - (c) 2004-2009. Progress Software Corporation. All rights reserved.

<metaInformation>
	<scenarios>
		<scenario default="yes" name="Scenario1" userelativepaths="yes" externalpreview="no" useresolver="yes" url="" outputurl="" processortype="datadirect" tcpport="0" profilemode="0" profiledepth="" profilelength="" urlprofilexml="" commandline=""
		          additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="6553701" user="" password="" validateoutput="no" validator="internal"
		          customvalidator="">
			<parameterValue name="xPRSXMLInput" value="doc('input.xml')"/>
			<advancedProperties name="DocumentURIResolver" value=""/>
			<advancedProperties name="CollectionURIResolver" value=""/>
			<advancedProperties name="ModuleURIResolver" value=""/>
		</scenario>
	</scenarios>
	<MapperMetaTag>
		<MapperInfo srcSchemaPathIsRelative="yes" srcSchemaInterpretAsXML="no" destSchemaPath="" destSchemaRoot="" destSchemaPathIsRelative="yes" destSchemaInterpretAsXML="no"/>
		<MapperBlockPosition></MapperBlockPosition>
		<TemplateContext></TemplateContext>
		<MapperFilter side="source"></MapperFilter>
	</MapperMetaTag>
</metaInformation>
:)